using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using System.IO;

namespace cssp.Test
{
	[TestFixture]
	public class ParserTest
	{
		CssxParser mParser;

		[SetUp]
		public void SetUp()
		{
			Log.Logger = new ConsoleLogger();
			mParser = new CssxParser();
			mParser.Scanner = new CssxScanner();
		}

		[TearDown]
		public void TearDown()
		{
		}

		[Test]
		public void TestSymbolDefinition()
		{
			string filename = "define.cssx";
			Unit unit = null;
			using (StreamReader sr = TestHelper.OpenTestFile(filename))
			{
				unit = mParser.Parse(filename, sr);
			}

			Assert.AreEqual(7, unit.Symbols.Count);
			Assert.AreEqual("test1", unit.Symbols[0].Name, "test1");
			Assert.AreEqual("", unit.Symbols[0].Value, "test1");
			Assert.AreEqual("test2", unit.Symbols[1].Name, "test2");
			Assert.AreEqual("two", unit.Symbols[1].Value, "test2");
			Assert.AreEqual("test3", unit.Symbols[2].Name, "test3");
			Assert.AreEqual("$test2, three", unit.Symbols[2].Value, "test3");
			Assert.AreEqual("test4", unit.Symbols[3].Name, "test4");
			Assert.AreEqual("\"", unit.Symbols[3].Value, "test4");
			Assert.AreEqual("test5", unit.Symbols[4].Name, "test5");
			Assert.AreEqual("\"\"", unit.Symbols[4].Value, "test5");
			Assert.AreEqual("test6", unit.Symbols[5].Name, "test6");
			Assert.AreEqual("one \"$test3, four\";", unit.Symbols[5].Value, "test6");
			Assert.AreEqual("test7", unit.Symbols[6].Name, "test7");
			Assert.AreEqual("$test6, five.", unit.Symbols[6].Value, "test7");
		}

		[Test]
		public void TestTarget()
		{
			Unit u;
			string filename = "target.cssx";
			using (StreamReader sr = TestHelper.OpenTestFile(filename))
			{
				u = mParser.Parse(filename, sr);
			}

			Assert.AreEqual(3, u.Targets.Keys.Count);
			Assert.AreEqual(3, u.Targets["ie6"].Count);
			Assert.AreEqual(1, u.Targets["ff2"].Count);
			Assert.AreEqual(1, u.Targets["l1"].Count);

			Assert.AreEqual(4, u.Targets["ie6"][0].StartLineNum);
			Assert.AreEqual(6, u.Targets["ie6"][0].EndLineNum);
			Assert.AreEqual(9, u.Targets["ie6"][1].StartLineNum);
			Assert.AreEqual(14, u.Targets["ie6"][1].EndLineNum);
			Assert.AreEqual(32, u.Targets["ie6"][2].StartLineNum);
			Assert.AreEqual(0, u.Targets["ie6"][2].EndLineNum);
			
			Assert.AreEqual(16, u.Targets["ff2"][0].StartLineNum);
			Assert.AreEqual(18, u.Targets["ff2"][0].EndLineNum);

			Assert.AreEqual(27, u.Targets["l1"][0].StartLineNum);
			Assert.AreEqual(29, u.Targets["l1"][0].EndLineNum);
		}
	}
}
