using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using System.IO;

namespace cssp.Test
{
	[TestFixture]
	public class ProcessorTest
	{
		CssxProcessor mProcessor;

		[SetUp]
		public void SetUp()
		{
			Log.Logger = new ConsoleLogger();
			mProcessor = new CssxProcessor();
		}

		[TearDown]
		public void TearDown()
		{
		}

		[Test]
		public void TestSymbolResolution()
		{
			Unit unit = mProcessor.Process(Path.Combine(TestHelper.TestPath, "process-define-valid.cssx"));

			Assert.AreEqual("", unit.Symbols[0].Value, "test1");
			Assert.AreEqual("Arial", unit.Symbols[1].Value, "test2");
			Assert.AreEqual("Arial, sans-serif", unit.Symbols[2].Value, "test3");
			Assert.AreEqual("\"", unit.Symbols[3].Value, "test4");
			Assert.AreEqual("\"\"", unit.Symbols[4].Value, "test5");
			Assert.AreEqual("\"Courier New\", Arial, sans-serif", unit.Symbols[5].Value, "test6");
			Assert.AreEqual("font-family: \"Courier New\", Arial, sans-serif; color:#336699", unit.Symbols[6].Value, "test7");
			Assert.AreEqual("#336699", unit.Symbols[7].Value, "test8");
			
		}

		[Test]
		public void TestImports()
		{
			Unit unit = mProcessor.Process(Path.Combine(TestHelper.TestPath, "process-import.cssx"));

			Assert.AreEqual(2, unit.Imports.Count, "# imports");
			StringReader sr = new StringReader(unit.Text);
			string line = sr.ReadLine();
			line = sr.ReadLine();
			line = sr.ReadLine();
			line = sr.ReadLine();

			Assert.AreEqual("arial", line, "i1_appFontFamily");
			line = sr.ReadLine();
			Assert.AreEqual("10pt", line, "i1_appFontSize");
			line = sr.ReadLine();
			Assert.AreEqual("black", line, "i1_appFontColor");
			line = sr.ReadLine();
			Assert.AreEqual("font-family:arial; font-size:10pt; color:black", line, "i1_appFont");
			line = sr.ReadLine();
			Assert.AreEqual("10pt", line, "i2_bodyFontSize");
			line = sr.ReadLine();
			Assert.AreEqual("i21_bgColor=background-color:#dfdfdf", line, "background");
			line = sr.ReadLine();
			Assert.AreEqual("background-color:#dfdfdf", line, "i21_bgColor");
		}
	}
}
