﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using System.IO;

namespace cssp.Test
{
	public class ScannerTestBase
	{
		protected CssxScanner mScanner;

		public ScannerTestBase()
		{
			mScanner = new CssxScanner();
		}
	}

	[TestFixture]
	public class ScannerTest : ScannerTestBase
	{
		int mNumValid, mNumInvalid;

		[SetUp]
		public void SetUp()
		{
		}

		[TearDown]
		public void TearDown()
		{
		}

		[Test]
		public void TestBasic()
		{
			mScanner.OnToken = OnTokenBasic;
			mNumValid = 0;
			using (StreamReader sr = TestHelper.OpenTestFile("scan-basic.cssx"))
			{
				mScanner.Scan(sr);
			}

			Assert.AreEqual(11, mNumValid);
		}

		void OnTokenBasic(Token[] tokens)
		{
			mNumValid += tokens.Length;

			foreach (Token tok in tokens)
			{
				switch (tok.Value)
				{
					case "token1":
						Assert.AreEqual(1, tok.Row, tok.Value + " row");
						Assert.AreEqual(1, tok.Col, tok.Value + " col");
						Assert.AreEqual(0, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.Word, tok.Type, tok.Value + " type");
						break;
					case "token2":
						Assert.AreEqual(2, tok.Row, tok.Value + " row");
						Assert.AreEqual(1, tok.Col, tok.Value + " col");
						Assert.AreEqual(8, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.Word, tok.Type, tok.Value + " type");
						break;
					case "token3":
						Assert.AreEqual(2, tok.Row, tok.Value + " row");
						Assert.AreEqual(8, tok.Col, tok.Value + " col");
						Assert.AreEqual(15, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.Word, tok.Type, tok.Value + " type");
						break;
					case "token4":
						Assert.AreEqual(3, tok.Row, tok.Value + " row");
						Assert.AreEqual(2, tok.Col, tok.Value + " col");
						Assert.AreEqual(24, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.Word, tok.Type, tok.Value + " type");
						break;
					case "token5":
						Assert.AreEqual(3, tok.Row, tok.Value + " row");
						Assert.AreEqual(11, tok.Col, tok.Value + " col");
						Assert.AreEqual(33, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.QuotedString, tok.Type, tok.Value + " type");
						break;
					case "token6":
						Assert.AreEqual(4, tok.Row, tok.Value + " row");
						Assert.AreEqual(2, tok.Col, tok.Value + " col");
						Assert.AreEqual(44, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.Word, tok.Type, tok.Value + " type");
						break;
					case "token7 token7":
						Assert.AreEqual(4, tok.Row, tok.Value + " row");
						Assert.AreEqual(9, tok.Col, tok.Value + " col");
						Assert.AreEqual(51, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.QuotedString, tok.Type, tok.Value + " type");
						break;
					case "token8":
						Assert.AreEqual(5, tok.Row, tok.Value + " row");
						Assert.AreEqual(1, tok.Col, tok.Value + " col");
						Assert.AreEqual(68, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.Word, tok.Type, tok.Value + " type");
						break;
					case "token9 \"token9\" token9":
						Assert.AreEqual(5, tok.Row, tok.Value + " row");
						Assert.AreEqual(8, tok.Col, tok.Value + " col");
						Assert.AreEqual(75, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.QuotedString, tok.Type, tok.Value + " type");
						break;
					case "token10":
						Assert.AreEqual(6, tok.Row, tok.Value + " row");
						Assert.AreEqual(1, tok.Col, tok.Value + " col");
						Assert.AreEqual(103, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.Word, tok.Type, tok.Value + " type");
						break;
					case "token11":
						Assert.AreEqual(10, tok.Row, tok.Value + " row");
						Assert.AreEqual(1, tok.Col, tok.Value + " col");
						Assert.AreEqual(150, tok.Offset, tok.Value + " offset");
						Assert.AreEqual(TokenType.Word, tok.Type, tok.Value + " type");
						break;
					default:
						Assert.Fail("Unexpected token: '" + tok.Value + "'");
						break;
				}
			}
		}

		[Test]
		public void TestInvalid()
		{
			mNumValid = mNumInvalid = 0;
			mScanner.OnToken = OnTokenInvalid;
			using (StreamReader sr = TestHelper.OpenTestFile("scan-invalid.cssx"))
			{
				mScanner.Scan(sr);
			}

			Assert.AreEqual(0, mNumValid, "Number of valid tokens");
			Assert.AreEqual(8, mNumInvalid, "Number of invalid tokens");
		}

		void OnTokenInvalid(Token[] tokens)
		{
			// --- invalid cases ---
			if (tokens.Length < 1)
				Assert.Fail("No tokens returned.");
			
			if (tokens[0].Value == "invalid1")
			{
				Assert.AreEqual(1, tokens.Length, tokens[0].Value);
				Assert.AreEqual(TokenType.Word, tokens[0].Type); 
				mNumInvalid++;
			}
			else if (tokens[0].Value == "invalid2")
			{
				Assert.AreEqual(2, tokens.Length, tokens[0].Value);
				Assert.AreEqual(TokenType.Word, tokens[0].Type);
				Assert.AreEqual(TokenType.Word, tokens[1].Type);
				Assert.AreEqual("\\\"", tokens[1].Value);
				mNumInvalid++;
			}
			else if (tokens[0].Value == "invalid3")
			{
				Assert.AreEqual(1, tokens.Length, tokens[0].Value);
				Assert.AreEqual(TokenType.Word, tokens[0].Type);
				mNumInvalid++;
			}
			else if (tokens[0].Value == "invalid4")
			{
				Assert.AreEqual(2, tokens.Length, tokens[0].Value);
				Assert.AreEqual(TokenType.Word, tokens[0].Type);
				Assert.AreEqual(TokenType.Word, tokens[1].Type);
				Assert.AreEqual("\\\"", tokens[1].Value);
				mNumInvalid++;
			}
			else if (tokens[0].Value == "invalid5")
			{
				Assert.AreEqual(2, tokens.Length, tokens[0].Value);
				Assert.AreEqual(TokenType.Word, tokens[0].Type);
				Assert.AreEqual(TokenType.Word, tokens[1].Type);
				Assert.AreEqual("\\\"\\\"", tokens[1].Value);
				mNumInvalid++;
			}
			else if (tokens[0].Value == "invalid6")
			{
				Assert.AreEqual(3, tokens.Length, tokens[0].Value);
				Assert.AreEqual(TokenType.Word, tokens[0].Type);
				Assert.AreEqual(TokenType.Word, tokens[1].Type);
				Assert.AreEqual(TokenType.Word, tokens[2].Type);
				mNumInvalid++;
			}
			else if (tokens[0].Value == "invalid7")
			{
				Assert.AreEqual(1, tokens.Length, tokens[0].Value);
				Assert.AreEqual(TokenType.Word, tokens[0].Type);
				mNumInvalid++;
			}
			else if (tokens[0].Value == "invalid8")
			{
				Assert.AreEqual(4, tokens.Length, tokens[0].Value);
				Assert.AreEqual(TokenType.Word, tokens[0].Type);
				Assert.AreEqual(TokenType.Word, tokens[1].Type);
				Assert.AreEqual(TokenType.Word, tokens[2].Type);
				Assert.AreEqual(TokenType.Word, tokens[3].Type);
				mNumInvalid++;
			}
		}

		[Test]
		public void TestValid()
		{
			mNumValid = mNumInvalid = 0;
			mScanner.OnToken = OnTokenValid;
			using (StreamReader sr = TestHelper.OpenTestFile("scan-valid.cssx"))
			{
				mScanner.Scan(sr);
			}

			Assert.AreEqual(15, mNumValid, "Number of valid tokens");			
		}

		void OnTokenValid(Token[] tokens)
		{
			if (tokens.Length == 2)
			{
				if (tokens[0].Value == "import")
					mNumValid++;
				
				if (tokens[1].Value == "file.cssi"
					|| tokens[1].Value == "file 1.cssi"
					|| tokens[1].Value == "file_a-1.0.cssi")
					mNumValid++;

			}
			else if (tokens.Length == 3)
			{
				if (tokens[0].Value == "define")
					mNumValid++;

				if (tokens[1].Value == "test1"
					|| tokens[1].Value == "test2"
					|| tokens[1].Value == "test3")
					mNumValid++;

				if (tokens[2].Value == ""
					|| tokens[2].Value == "one \"$test3, four\";"
					|| tokens[2].Value == "$test6, five.")
					mNumValid++;
			}
		}
	}
}
